import re

from django_enumfield import enum
from rest_framework import serializers
from rest_framework.utils.field_mapping import get_field_kwargs, ClassLookupDict
import base32_crockford


class Base32CrockfordField(serializers.CharField):

    def to_internal_value(self, data):
        data = super(Base32CrockfordField, self).to_internal_value(data)
        try:
            return base32_crockford.normalize(data)
        except ValueError:
            raise serializers.ValidationError('Invalid {}'.format(self.source))


class USPhoneNumberField(serializers.CharField):

    def to_internal_value(self, data):
        data = re.sub('[^0-9]', '', data)
        data_length = len(data)

        if 10 > data_length > 11:
            raise serializers.ValidationError('The {} should be 10 to 11 numbers in length.'.format(self.source))

        # Prepend the country code
        if data_length == 10:
            data = '+1{}'.format(data)
        else:
            data = '+{}'.format(data)

        return data


class DateTimeSerializerMixin(serializers.Serializer):

    created_at = serializers.DateTimeField(read_only=True)
    updated_at = serializers.DateTimeField(read_only=True)


class PrimaryKeyMixin(serializers.ModelSerializer):
    id = serializers.CharField(read_only=True)


class WritableIDSerializer(serializers.Serializer):
    id = serializers.CharField()


class EnumField(serializers.ChoiceField):
    """ A field that takes a field's value as the key and returns
    the associated value for serialization """
    def __init__(self, choices, *args, **kwargs):
        self.representations = {}
        self.internal_values = {}

        for k, v in choices:
            if v in self.internal_values:
                raise ValueError(
                    'The field is not deserializable with the given choices.'
                    ' Please ensure that choices map 1:1 with values'
                )
            self.internal_values[v.name] = k
            self.representations[k] = v.name

        super(EnumField, self).__init__(self.internal_values, **kwargs)

    def to_internal_value(self, data):
        res = super(EnumField, self).to_internal_value(data)
        return self.internal_values.get(data) if res else 3

    def to_representation(self, value):
        return self.representations.get(value)


class Serializer(PrimaryKeyMixin, serializers.Serializer):

    def __init__(self, *args, **kwargs):
        nested_write = kwargs.pop('nested_write', False)
        if nested_write:
            for k, field in self.fields.items():
                if field.field_name == 'id':
                    field.read_only = False
                else:
                    field.read_only = True
        super(Serializer, self).__init__(*args, **kwargs)

    def build_standard_field(self, field_name, model_field):
        field_mapping = ClassLookupDict(self.serializer_field_mapping)

        field_class = field_mapping[model_field]
        field_kwargs = get_field_kwargs(field_name, model_field)

        if not issubclass(field_class, serializers.ModelField):
            # `model_field` is only valid for the fallback case of
            # `ModelField`, which is used when no other typed field
            # matched to the model field.
            field_kwargs.pop('model_field', None)

        if field_class == EnumField:
            return field_class, field_kwargs
        else:
            return super(Serializer, self).build_standard_field(field_name, model_field)


class ModelSerializer(PrimaryKeyMixin, serializers.ModelSerializer):
    serializer_field_mapping = serializers.ModelSerializer.serializer_field_mapping
    serializer_field_mapping[enum.EnumField] = EnumField

    def __init__(self, *args, **kwargs):
        nested_write = kwargs.pop('nested_write', False)
        if nested_write:
            for k, field in self.fields.items():
                if field.field_name == 'id':
                    field.read_only = False
                else:
                    field.read_only = True
        super(ModelSerializer, self).__init__(*args, **kwargs)

    def get_default_field_names(self, declared_fields, model_info):
        """
        Return the default list of field names that will be used if the
        `Meta.fields` option is not specified.
        """
        return (
            [model_info.pk.name] +
            [self.url_field_name] +
            list(declared_fields.keys()) +
            list(model_info.fields.keys()) +
            list(model_info.forward_relations.keys())
        )

    def build_standard_field(self, field_name, model_field):
        field_mapping = ClassLookupDict(self.serializer_field_mapping)

        field_class = field_mapping[model_field]
        field_kwargs = get_field_kwargs(field_name, model_field)

        if not issubclass(field_class, serializers.ModelField):
            # `model_field` is only valid for the fallback case of
            # `ModelField`, which is used when no other typed field
            # matched to the model field.
            field_kwargs.pop('model_field', None)

        if field_class == EnumField:
            return field_class, field_kwargs
        else:
            return super(ModelSerializer, self).build_standard_field(field_name, model_field)


class HyperlinkedModelSerializer(serializers.HyperlinkedModelSerializer):
    serializer_field_mapping = serializers.HyperlinkedModelSerializer.serializer_field_mapping
    serializer_field_mapping[enum.EnumField] = EnumField

    def build_standard_field(self, field_name, model_field):
        field_mapping = ClassLookupDict(self.serializer_field_mapping)

        field_class = field_mapping[model_field]
        field_kwargs = get_field_kwargs(field_name, model_field)

        if not issubclass(field_class, serializers.ModelField):
            # `model_field` is only valid for the fallback case of
            # `ModelField`, which is used when no other typed field
            # matched to the model field.
            field_kwargs.pop('model_field', None)

        if field_class == EnumField:
            return field_class, field_kwargs
        else:
            return super(HyperlinkedModelSerializer, self).build_standard_field(field_name, model_field)



from dateutil.relativedelta import relativedelta
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers, exceptions
import datetime


class AtLeastOneRequiredValidator(object):
    message = _('At least one of the fields ({}) is required.')

    def __init__(self, *fields):
        self.fields = fields

    def __call__(self, attrs):
        passes = False
        for field in self.fields:
            if attrs.get(field):
                passes = True
                break

        if not passes:
            fields_str = ', '.join(self.fields)
            raise serializers.ValidationError(self.message.format(fields_str))


class AtLeastAge(object):
    message = _('Need to be at least {} years of age.')

    def __init__(self, age, raises=exceptions.PermissionDenied):
        self.age = age
        self.raises = raises

    def __call__(self, value):
        age = relativedelta(datetime.date.today(), value)

        if age.years < self.age:
            raise self.raises(self.message.format(self.age))




from django.contrib.auth import get_user_model
from django.core.exceptions import PermissionDenied
from rest_framework import permissions


class RequirePassword(permissions.BasePermission):
    """
    When this permission is added to a view[set] it requires that the
    `password` field also be passed as part of the request, useful for change email/password type requests
    where you want to make sure the request actually has permission.
    """
    def has_permission(self, request, view):
        password = request.data.get('password')
        if not password or not request.user.check_password(password):
            raise PermissionDenied

        return True


class IsAuthenticatedOrReadOnlyOrCreate(permissions.BasePermission):
    """
    The request is authenticated as a user, or is a read-only request, or non-authenticated create
    """
    def has_permission(self, request, view):
        if (request.method in ['GET', 'HEAD', 'OPTIONS'] or
                (request.user and request.user.is_authenticated()) or
                    request.method == 'POST'):
            return True
        return False


class IsOwner(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has the attribute specified as attr, otherwise
    it assumes the object itself is a User object to compare to authenticated User.
    """
    def __init__(self, user_attr=None, actions=('create', 'list')):
        self.user_attr = user_attr
        self.actions = actions

    def __call__(self):
        return self

    def has_permission(self, request, view):
        if view.action in self.actions:
            user_id = view.kwargs.get(self.user_attr) if self.user_attr else view.kwargs.get('parent_lookup_user')
            return str(user_id) == str(request.user.pk)

        return True

    def has_object_permission(self, request, view, obj):

        if self.user_attr:
            user_or_user_id = getattr(obj, self.user_attr)
            if isinstance(user_or_user_id, get_user_model()):
                return user_or_user_id == request.user
            else:
                return str(user_or_user_id) == str(request.user.pk)
        else:
            return obj == request.user



from rest_framework.pagination import PageNumberPagination


class LargePageNumberPagination(PageNumberPagination):
    page_size = 100


class FlexPageSizePagination(PageNumberPagination):
    page_size = 10
    page_size_query_param = 'page_size'
    max_page_size = 100



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='fh-drf-common',
    version='0.1.14',
    packages=['rest_framework_common'],
    include_package_data=True,
    description='A library to provide common functionality for Django Rest Framework projects.',
    long_description=README,
    url='https://gitlab.com/futurehaus/drf-common',
    install_requires=['base32-crockford', 'six', 'django-enumfield', 'python-dateutil']
)
